/* Copyright (C) 1994, 1995 Aladdin Enterprises.  All rights reserved.
  
  This file is part of Aladdin Ghostscript.
  
  Aladdin Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author
  or distributor accepts any responsibility for the consequences of using it,
  or for whether it serves any particular purpose or works at all, unless he
  or she says so in writing.  Refer to the Aladdin Ghostscript Free Public
  License (the "License") for full details.
  
  Every copy of Aladdin Ghostscript must include a copy of the License,
  normally in a plain ASCII text file named PUBLIC.  The License grants you
  the right to copy, modify and redistribute Aladdin Ghostscript, but only
  under certain conditions described in the License.  Among other things, the
  License requires that the copyright notice and this notice be preserved on
  all copies.
*/


#ifndef gdevmac_INCLUDED
#  define gdevmac_INCLUDED


#include "gx.h"				/* for gx_bitmap; includes std.h */
#include "gxdevice.h"

#define DEFAULT_PAGE_SIZE_H		8.5
#define DEFAULT_PAGE_SIZE_V		11.0
#define DEFAULT_DEV_DPI			72.0

#define DEFAULT_DEV_WIDTH  		(DEFAULT_PAGE_SIZE_H * DEFAULT_DEV_DPI)
#define DEFAULT_DEV_HEIGHT 		(DEFAULT_PAGE_SIZE_V * DEFAULT_DEV_DPI)
#define DEFAULT_PAGE_WIDTH  	(DEFAULT_PAGE_SIZE_H * 72.0)
#define DEFAULT_PAGE_HEIGHT 	(DEFAULT_PAGE_SIZE_V * 72.0)

//#define MAC_DEV_DEBUG 1
//#define WIN_PALETTE 1
extern CTabHandle		bw_clut;
extern ColorSpecPtr		bw_colors;
extern CTabHandle		gray_4bit_clut;
extern ColorSpecPtr		gray_4bit_colors;
extern CTabHandle		mac_8bit_clut;
extern ColorSpecPtr		mac_8bit_colors;

void init_palette(void);

typedef struct draw_port_s
{
	struct draw_port_s	* next;				/* list of open ports */
	
	CGrafPtr		port;					/* the port to draw into */
	Boolean			isGWorld;				/* port is a GWorld */
	Boolean			setOriginAndClip;		/* true if we are to use origin and clipping */
	Point			origin;					/* device coord of top left of port content */
	Rect			viewRect;				/* drawable area of port in QD local coords */
	gx_color_index	lastForeColor;			/* the last color we set */
} draw_port;



// Note: this structure and its values
// are part of the preferences... 
// Do not change willy nilly!

typedef struct dev_spec_s {
	float	width, height;		// always in points (1/72 of an inch)
	char	units;
	char	ignore_comments;
	float	pixels_per_inch;
	char	scale;
	char	depth;
} dev_spec;

#define SPEC_UNITS_INCH		0
#define SPEC_UNITS_CM		1
#define SPEC_UNITS_POINT	2

#define SPEC_SCALE_PERCENT	0
#define SPEC_SCALE_TIMES	1
#define SPEC_SCALE_DIVIDE	2
#define SPEC_SCALE_DPI		3

#define SPEC_DEPTH_MONO			0
#define SPEC_DEPTH_4BIT_GRAY	1
#define SPEC_DEPTH_8BIT_GRAY	2
#define SPEC_DEPTH_4BIT_COLOR	3
#define SPEC_DEPTH_8BIT_COLOR	4
#define SPEC_DEPTH_24BIT_COLOR	5



typedef struct mac_gs_win_s {
	struct mac_gs_win_s		* next;
	struct gx_device_mac_s	* mdev;
	
	draw_port		win;
	draw_port		pict;
	
	Boolean			pixmapLocked;
	PixMapHandle	pixmap;
	
	ControlHandle	hScrollBar;
	ControlHandle	vScrollBar;
	
	Boolean			pageOutput;				/* page has been output */
	Boolean			delayedClear;			/* page needs clear on next draw */
	Boolean			initiallyVisible;		/* was the page initially visible */
} mac_gs_win;


extern dev_spec		nextSpec;
extern mac_gs_win	* currWindow;			/* most receint rendering window */


/* Define the Macintosh device */

typedef struct gx_device_mac_s
{
	gx_device_common;
	mac_gs_win		* mwin;
	Boolean			visible;				/* show when created, even if blank */
	Str255			title;					/* window title */
	CTabHandle		clut;
	ColorSpecPtr	colors;
} gx_device_mac;

#define DEV_MAC_NAME	"mac"


/* Inteface for manipulating mac_gs_win */

mac_gs_win *macw_create_window	(gx_device_mac *mdev);
void		macw_close_window   (mac_gs_win *mwin);
void		macw_clear_window   (mac_gs_win *mwin);
mac_gs_win *macw_find_win(WindowPtr win);

void		macw_top_window		(mac_gs_win *mwin);
void		macw_detach_window	(mac_gs_win *mwin);

void		macw_close_picture  (mac_gs_win *mwin);
void		macw_clear_picture  (mac_gs_win *mwin);
void		macw_copy_picture   (mac_gs_win *mwin);
void		macw_save_picture	(mac_gs_win *mwin);

void		macw_inval_window   (mac_gs_win *mwin);
void		macw_redraw_window  (mac_gs_win *mwin);
void		macw_resize_window  (mac_gs_win *mwin);


#endif 				/* gdevmac_INCLUDED */
