/* Copyright (C) 1994, 1995 Aladdin Enterprises.  All rights reserved.
  
  This file is part of Aladdin Ghostscript.
  
  Aladdin Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author
  or distributor accepts any responsibility for the consequences of using it,
  or for whether it serves any particular purpose or works at all, unless he
  or she says so in writing.  Refer to the Aladdin Ghostscript Free Public
  License (the "License") for full details.
  
  Every copy of Aladdin Ghostscript must include a copy of the License,
  normally in a plain ASCII text file named PUBLIC.  The License grants you
  the right to copy, modify and redistribute Aladdin Ghostscript, but only
  under certain conditions described in the License.  Among other things, the
  License requires that the copyright notice and this notice be preserved on
  all copies.
*/

#include "MacHeaders"
#include "Palettes.h"
#include "QDOffscreen.h"
#include <stdio.h>
#include <stddef.h>
#include <string.h>
#include "gdevmac.h"
#include "gp_mac.h"

static mac_gs_win	* theGSWins = NULL;
mac_gs_win			* currWindow = NULL;

static short		newTop = 0, newLeft = 0;


#define mac_8bit_clut_resource		300

CTabHandle		bw_clut				= NULL;
ColorSpecPtr	bw_colors;
CTabHandle		gray_4bit_clut		= NULL;
ColorSpecPtr	gray_4bit_colors;
CTabHandle		mac_8bit_clut		= NULL;
ColorSpecPtr	mac_8bit_colors;

#ifdef WIN_PALETTE
PaletteHandle	mac_8bit_palette	= NULL;
#endif

void init_palette(void)
{
	long					s;
	register int			i;
	register long			r, g, b;
	register long			v, w;
	register ColorSpec		* spec;
	
	s = sizeof(ColorTable) + 2 * sizeof(ColorSpec);
	bw_clut = (CTabHandle)NewHandle(s);
	if (bw_clut == NULL) noMemoryExit();
	MoveHHi((Handle)bw_clut);
	HLock((Handle)bw_clut);
	memset(*bw_clut, 0, s);

	(*bw_clut)->ctSeed = GetCTSeed();
	(*bw_clut)->ctFlags = 0x8000;
	(*bw_clut)->ctSize = 1;
	
	(*bw_clut)->ctTable[0].rgb.red = 65535;
	(*bw_clut)->ctTable[0].rgb.green = 65535;
	(*bw_clut)->ctTable[0].rgb.blue = 65535;

	(*bw_clut)->ctTable[1].rgb.red = 0;
	(*bw_clut)->ctTable[1].rgb.green = 0;
	(*bw_clut)->ctTable[1].rgb.blue = 0;
		
	bw_colors = (*bw_clut)->ctTable;


	s = sizeof(ColorTable) + 16 * sizeof(ColorSpec);
	gray_4bit_clut = (CTabHandle)NewHandle(s);
	if (gray_4bit_clut == NULL) noMemoryExit();
	MoveHHi((Handle)gray_4bit_clut);
	HLock((Handle)gray_4bit_clut);
	memset(*gray_4bit_clut, 0, s);
	
	(*gray_4bit_clut)->ctSeed = GetCTSeed();
	(*gray_4bit_clut)->ctFlags = 0x8000;
	(*gray_4bit_clut)->ctSize = 15;
	
	spec = (*gray_4bit_clut)->ctTable;
	for (w = 65535; w >= 0; w -= (65535 / 15)) {
		spec->rgb.red = w;
		spec->rgb.green = w;
		spec->rgb.blue = w;
		
		spec += 1;
	}
	
	gray_4bit_colors = (*gray_4bit_clut)->ctTable;


	s = sizeof(ColorTable) + 256 * sizeof(ColorSpec);
	mac_8bit_clut = (CTabHandle)NewHandle(s);
	if (mac_8bit_clut == NULL) noMemoryExit();
	MoveHHi((Handle)mac_8bit_clut);
	HLock((Handle)mac_8bit_clut);
	memset(*mac_8bit_clut, 0, s);
	
	(*mac_8bit_clut)->ctSeed = GetCTSeed();
	(*mac_8bit_clut)->ctFlags = 0x8000;
	(*mac_8bit_clut)->ctSize = 255;
	
	i = 0;
	spec = (*mac_8bit_clut)->ctTable;
	for (r = 65535; r >= 0; r -= (65535 / 5)) {
		for (g = 65535; g >= 0; g -= (65535 / 5)) {
			for (b = 65535; b >= 0; b -= (65535 / 5)) {
				spec->rgb.red = r;
				spec->rgb.green = g;
				spec->rgb.blue = b;
				
				i += 1;
				spec += 1;
				
				if (i == 215) goto do_ramps;
			}
		}
	}
do_ramps:
	for (v = 65535; v >= (65535 / 5); v -= (65535 / 5)) {
		w = v - (65535 / 15);
		
		spec[0].rgb.red = w;
		spec[0].rgb.green = 0;
		spec[0].rgb.blue = 0;
		
		spec[10].rgb.red = 0;
		spec[10].rgb.green = w;
		spec[10].rgb.blue = 0;

		spec[20].rgb.red = 0;
		spec[20].rgb.green = 0;
		spec[20].rgb.blue = w;
		
		spec[30].rgb.red = w;
		spec[30].rgb.green = w;
		spec[30].rgb.blue = w;
		
		w = v - ((65535 / 15) * 2);
		
		spec[1].rgb.red = w;
		spec[1].rgb.green = 0;
		spec[1].rgb.blue = 0;
		
		spec[11].rgb.red = 0;
		spec[11].rgb.green = w;
		spec[11].rgb.blue = 0;

		spec[21].rgb.red = 0;
		spec[21].rgb.green = 0;
		spec[21].rgb.blue = w;
		
		spec[31].rgb.red = w;
		spec[31].rgb.green = w;
		spec[31].rgb.blue = w;
		
		spec += 2;
		i += 2;
	}
	spec += 30;
	i += 30;
	spec->rgb.red = 0;
	spec->rgb.green = 0;
	spec->rgb.blue = 0;

	mac_8bit_colors = (*mac_8bit_clut)->ctTable;


#ifdef WIN_PALETTE
	mac_8bit_palette = NewPalette(8, mac_8bit_clut, pmAnimated, 0x0000);
	if (mac_8bit_palette == NULL) noMemoryExit();
#endif
}


mac_gs_win *
macw_create_window(gx_device_mac * mdev)
{
	mac_gs_win	* mwin;
    Rect		windowRect;
    CGrafPtr	savePort;
    GDHandle	saveDev;
	short		width;
	short		height;
	OSErr		err;

	
	if (mac_8bit_clut == NULL)
		init_palette();

	mwin = NULL;
	
	mwin = (mac_gs_win *)NewPtr(sizeof(mac_gs_win));
	if (mwin == NULL) goto error;
	memset(mwin, 0, sizeof(mwin));
	
	width = mdev->width;
	height = mdev->height;

	/*...Create a color graphics window...*/

	SetRect(&windowRect, 0, 0, width + 15, height + 15);
	OffsetRect(&windowRect, newLeft + 4, newTop + 22 + LMGetMBarHeight());
	if (windowRect.bottom > (gHeight - 3)) windowRect.bottom = gHeight - 3;
	if (windowRect.right > (gWidth - 3)) windowRect.right = gWidth - 3;
	
	newLeft = (newLeft + 12) % 84;
	newTop = (newTop + 16) % 84;
	
	GetGWorld(&savePort, &saveDev);

	mwin->win.port = (CGrafPtr)NewCWindow((Ptr)NULL, &windowRect,
						mdev->title, FALSE, zoomDocProc, (WindowPtr)-1, TRUE, 0L);
	if (mwin->win.port == NULL) goto error;
	
#ifdef WIN_PALETTE
	SetPalette(mwin->win.port, mac_8bit_palette, TRUE);
#endif

	mwin->win.isGWorld = false;
	mwin->win.setOriginAndClip = true;
	mwin->win.lastForeColor = gx_no_color_index;
	

	SetPort((GrafPtr)mwin->win.port);

	/*...Create the scrollbars...*/
	mwin->hScrollBar = NewControl((GrafPtr)mwin->win.port,
				&mwin->win.viewRect, "\p", FALSE, 0, 0, 0, scrollBarProc, (long)mwin);
	mwin->vScrollBar = NewControl((GrafPtr)mwin->win.port,
				&mwin->win.viewRect, "\p", FALSE, 0, 0, 0, scrollBarProc, (long)mwin);
	if (mwin->hScrollBar == NULL  ||  mwin->vScrollBar == NULL) goto error;
	
	mwin->win.origin.h = 0;
	mwin->win.origin.v = 0;
	SetRect(&mwin->pict.viewRect, 0, 0, width, height);
	
	/*...Position the scrollbars and calculate the view port's size...*/
	macw_resize_window(mwin);
	ValidRect(&mwin->win.viewRect);

	/*...Show, but do not activate, the scrollbars...*/
	ShowControl(mwin->hScrollBar);
	HiliteControl(mwin->hScrollBar, 255);
	ShowControl(mwin->vScrollBar);
	HiliteControl(mwin->vScrollBar, 255);

	/* Set Zoom limit to be current maximal size */
	((WStateData *)*(((WindowPeek)mwin->win.port)->dataHandle))->stdState
		= windowRect;


	/*...Create an "offscreen" window used to capture the rendered picture...*/
	{
		CTabHandle		the_clut;
		
		switch (mdev->color_info.depth) {
			case 1:	the_clut = bw_clut;			break;
			case 4:	the_clut = gray_4bit_clut;	break;
			default:
			case 8:	the_clut = mac_8bit_clut;	break;
		}
		
		err = NewGWorld(&mwin->pict.port,
				mdev->color_info.depth,
				&mwin->pict.viewRect,
				the_clut, NULL, 0);
		if (err != noErr) goto error;	// most likely source of errors!
	}
	
	mwin->pixmap = GetGWorldPixMap(mwin->pict.port);
	mwin->pict.isGWorld = true;
	
	LockPixels(mwin->pixmap);
	mwin->pixmapLocked = true;

	macw_clear_picture(mwin);

	mwin->pict.setOriginAndClip = false;
	mwin->pict.lastForeColor = gx_no_color_index;

	SetGWorld(savePort, saveDev);

	mwin->win.next = &mwin->pict;
	mwin->pict.next = NULL;
	mwin->pageOutput = false;
	mwin->delayedClear = false;
	mwin->initiallyVisible = mdev->visible;	//HACK!
	
	mwin->mdev = mdev;
	mdev->mwin = mwin;
	currWindow = mwin;
	
	mwin->next = theGSWins;
	theGSWins = mwin;
	
	ShowWindow((WindowPtr)mwin->win.port);
	
	return mwin;

error:
	if (mwin != NULL) {
		if (mwin->win.port != NULL)
			DisposeWindow((WindowPtr)mwin->win.port);
			
		DisposePtr((Ptr)mwin);
	}
	Alert(308, NULL);
	return NULL;
}


void
macw_clear_window(mac_gs_win *mwin)
{
	GrafPtr		savePort;
	
	GetPort(&savePort);
	SetPort((GrafPtr)mwin->win.port);
	
	BackColor(whiteColor);
	EraseRect(&mwin->win.viewRect);
	
	SetPort(savePort);
	
	
	macw_clear_picture(mwin);
}


void
macw_close_window(mac_gs_win *mwin)
{
	if (currWindow == mwin)
		currWindow = NULL;
		
	macw_close_picture(mwin);
	
	if (mwin->win.port /* != (GrafPtr) NULL */)
	{
		if (mwin->pict.port != (CGrafPtr)NULL) {
			DisposeGWorld(mwin->pict.port);
		}

		DisposeWindow((WindowPtr)mwin->win.port);
		mwin->win.port = (CGrafPtr) NULL;
	}

	if (theGSWins == mwin)
		theGSWins = mwin->next;
	else {
		mac_gs_win	* x;
		
		for (x = theGSWins; x != NULL; x = x->next)
			if (x->next == mwin) {
				x->next = mwin->next;
				break;
			}
	}
	mwin->next = NULL;
	
	DisposePtr((Ptr)mwin);
}


mac_gs_win *
macw_find_win(WindowPtr window)
{
	mac_gs_win	* mwin;
	
	for (mwin = theGSWins; mwin != NULL; mwin = mwin->next)
		if (mwin->win.port == (CGrafPtr)window)
			break;
	
	return mwin;
}




void
macw_top_window(mac_gs_win *mwin)
{
	ShowWindow((WindowPtr)mwin->win.port);
	SelectWindow((WindowPtr)mwin->win.port);
}


void
macw_detach_window(mac_gs_win *mwin)
{
	/* logically this should clear out currWindow if it is, but the
		rendering window command works funny if you do */
		
	macw_close_picture(mwin);
	
	if (mwin->mdev != NULL) {
		mwin->mdev->mwin = NULL;
		mwin->mdev = NULL;
	}
}


void
macw_clear_picture(mac_gs_win *mwin)

{
	GWorldPtr		savePort;
	GDHandle		saveDev;
	
	GetGWorld(&savePort, &saveDev);
	SetGWorld(mwin->pict.port, nil);
	
	if (!mwin->pixmapLocked) LockPixels(mwin->pixmap);
	
	BackColor(whiteColor);
	EraseRect(&mwin->pict.viewRect);
	
	if (!mwin->pixmapLocked) UnlockPixels(mwin->pixmap);
	
	SetGWorld(savePort, saveDev);
}

void
macw_close_picture(mac_gs_win *mwin)
{
	UnlockPixels(mwin->pixmap);
	mwin->pixmapLocked = false;
}



static PicHandle
macw_make_picture(mac_gs_win *mwin)
{
	CGrafPtr		savePort;
	GDHandle		saveDev;
	Rect			viewRect;
	PicHandle		ph;
	

	GetGWorld(&savePort, &saveDev);
	SetGWorld(mwin->pict.port, NULL);
	
	viewRect = mwin->pict.viewRect;
	ClipRect(&viewRect);
	ph = OpenPicture(&viewRect);

	if (!mwin->pixmapLocked) LockPixels(mwin->pixmap);
	
	ForeColor(blackColor);
	BackColor(whiteColor);
	CopyBits((BitMap *)*mwin->pixmap, (BitMap *)*mwin->pixmap,
		 &viewRect, &viewRect, srcCopy, nil);
		 
	if (!mwin->pixmapLocked) UnlockPixels(mwin->pixmap);
	
	ClosePicture();
	
	SetGWorld(savePort, saveDev);
	
	if (EmptyRect(&(*ph)->picFrame)) {
		KillPicture(ph);
		return NULL;
	}
	
	return ph;
}

void
macw_copy_picture(mac_gs_win *mwin)

{
	PicHandle	ph;
	
	ph = macw_make_picture(mwin);
	if (ph == NULL) return;
	
	ZeroScrap();

	HLock((Handle)ph);
	PutScrap(GetHandleSize((Handle)ph), 'PICT', *(Handle)ph);
	HUnlock((Handle)ph);
	
	KillPicture(ph);
}


void
macw_save_picture(mac_gs_win *mwin)
{
	StandardFileReply		reply;
	short					ref = -1;
	long					count;
	int						i;
	unsigned char			header[512];
	Str63					name;
	OSErr					err;
	PicHandle				ph;
	
	ph = macw_make_picture(mwin);
	if (ph == NULL) return;
	
	GetWTitle((WindowPtr)mwin->win.port, name);
	for (i = name[0]; i > 0; i--) {
		if (name[i] == '.') {
			if (i < (63 - 4)) {
				name[0] = i + 4;
				memcpy(&name[i+1], "pict", 4);
			}
			else {
				name[0] = i - 1;
			}
			break;
		}
	}
	
	StandardPutFile("\pSave PICT as", name, &reply);
	
	if (!reply.sfGood) return;
	
	err = FSpCreate(&reply.sfFile, 'ttxt', 'PICT', reply.sfScript);
	if (err == dupFNErr) {
		FSpDelete(&reply.sfFile);
		err = FSpCreate(&reply.sfFile, 'ttxt', 'PICT', reply.sfScript);
	}
	if (err != noErr) goto error;
	
	err = FSpOpenDF(&reply.sfFile, fsWrPerm, &ref);
	if (err != noErr) goto error;
	
	/* write the header: 512 bytes of zero */
	memset(header, 0, sizeof(header));
	count = 512;
	err = FSWrite(ref, &count, header);
	if (err != noErr) goto error;
	
	/* write the PICT */
	HLock((Handle)ph);
	count = GetHandleSize((Handle)ph);
	err = FSWrite(ref, &count, *ph);
	HUnlock((Handle)ph);
	
	KillPicture(ph);
	ph = NULL;

	FSClose(ref);
	return;
	
error:
	if (ph)
		KillPicture(ph);
	if (ref != -1)
		FSClose(ref);
	SysBeep(1);
}



/*
 *	Invalidates the scrollbar and drag icon areas.
 */
 
void
macw_inval_window(mac_gs_win *mwin)

{
	Rect			invalRect;
	GrafPtr			savePort;

	GetPort(&savePort);
	SetPort((GrafPtr)mwin->win.port);
	
	HideControl(mwin->hScrollBar);
	HideControl(mwin->vScrollBar);

	invalRect  = mwin->win.port->portRect;
	invalRect.left = invalRect.right  - 15;
	invalRect.top  = invalRect.bottom - 15;
	EraseRect(&invalRect);
	InvalRect(&invalRect);

	SetPort(savePort);
}


/*
 *	Redraws the grow icon and scrollbars.
 */

void
macw_redraw_window(mac_gs_win *mwin)
{
	GrafPtr		savePort;
	Rect		viewRect, pictRect;
	
	GetPort(&savePort);
	SetPort((GrafPtr)mwin->win.port);
	
	macw_resize_window(mwin);
	
	/*...Rescroll current offset if needed... */
	
	viewRect = mwin->win.viewRect;
	pictRect = mwin->pict.viewRect;
	
	if (mwin->win.origin.h + viewRect.right > pictRect.right) {
		mwin->win.origin.h = pictRect.right - viewRect.right;
		InvalRect(&viewRect);
	}
	SetCtlValue(mwin->hScrollBar, mwin->win.origin.h);
	
	if (mwin->win.origin.v + viewRect.bottom > pictRect.bottom) {
		mwin->win.origin.v = pictRect.bottom - viewRect.bottom;
		InvalRect(&viewRect);
	}
	SetCtlValue(mwin->vScrollBar, mwin->win.origin.v);

	DrawGrowIcon((WindowPtr)mwin->win.port);
	ShowControl(mwin->vScrollBar);
	ShowControl(mwin->hScrollBar);
		
	SetPort(savePort);
}


static void
macw_resize_window(mac_gs_win *mwin)

{
	Rect			portRect = mwin->win.port->portRect;
	short			wide;
	short			high;


	/*...Reposition and resize the scrollbars...*/

	MoveControl(mwin->vScrollBar, portRect.right - 15, portRect.top - 1);
	wide = 16;
	high = (portRect.bottom - portRect.top) - 13;
	SizeControl(mwin->vScrollBar, wide, high);

	MoveControl(mwin->hScrollBar, portRect.left - 1, portRect.bottom - 15);
	wide = (portRect.right - portRect.left) - 13;
	high = 16;
	SizeControl(mwin->hScrollBar, wide, high);

	/*...Recalculate the view port's rectangle...*/

	portRect.right  -= 15;			/*	adjustment for scrollbars	*/
	portRect.bottom -= 15;
	mwin->win.viewRect = portRect;

	/*...Reset the scrollbars' maxima...*/

	wide = portRect.right - portRect.left;
	SetCtlMax(mwin->hScrollBar, mwin->pict.viewRect.right - wide);

	high = portRect.bottom - portRect.top;
	SetCtlMax(mwin->vScrollBar, mwin->pict.viewRect.bottom - high);
}

