/* Copyright (C) 1994, 1995 Aladdin Enterprises.  All rights reserved.
  
  This file is part of Aladdin Ghostscript.
  
  Aladdin Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author
  or distributor accepts any responsibility for the consequences of using it,
  or for whether it serves any particular purpose or works at all, unless he
  or she says so in writing.  Refer to the Aladdin Ghostscript Free Public
  License (the "License") for full details.
  
  Every copy of Aladdin Ghostscript must include a copy of the License,
  normally in a plain ASCII text file named PUBLIC.  The License grants you
  the right to copy, modify and redistribute Aladdin Ghostscript, but only
  under certain conditions described in the License.  Among other things, the
  License requires that the copyright notice and this notice be preserved on
  all copies.
*/

#include "MacHeaders"
#include <Folders.h>
#include <stdio.h>
#include <string.h>
#include "math_.h"
#include "gdevmac.h"
#include "gp_mac.h"



#define spec_resource		302
#define outfile_resource	303
#define confirm_resource	304
#define about_resource		305
#define pref_resource		306

#define spec_help_resource		1302
#define outfile_help_resource	1303
#define confirm_help_resource	1304
#define about_help_resource		1305
#define pref_help_resource		1306


#define animation_resource	305

enum {
	sitem_ok					= 1,
	sitem_cancel				= 2,


	sitem_size_h				= 3,
	sitem_size_v				= 4,
	
	sitem_first_unit			= 5,
	sitem_inch_unit				= 5,
	sitem_cm_unit				= 6,
	sitem_point_unit			= 7,
	sitem_last_unit				= 7,
		// the above must be in same order as SPEC_UNITS_XXX
		sitem_unit_offset		= (sitem_inch_unit - SPEC_UNITS_INCH),
		
	sitem_size_popup			= 8,
	
	sitem_ignore_comments		= 9,
	
	
	sitem_scale_amount			= 10,
	
	sitem_first_scale			= 11,
	sitem_percent_scale			= 11,
	sitem_times_scale			= 12,
	sitem_divide_scale			= 13,
	sitem_dpi_scale				= 14,
	sitem_last_scale			= 14,
		// the above must be in same order as SPEC_SCALE_XXX
		sitem_scale_offset		= (sitem_percent_scale - SPEC_SCALE_PERCENT),
		
	sitem_first_depth			= 15,
	sitem_mono_depth			= 15,
	sitem_4_bit_gray_depth		= 16,
	sitem_8_bit_gray_depth		= 17,
	sitem_4_bit_color_depth		= 18,
	sitem_8_bit_color_depth		= 19,
	sitem_24_bit_color_depth	= 20,
	sitem_last_depth			= 20,
		// the above must be in same order as SPEC_DEPTH_XXX
		sitem_depth_offset		= (sitem_mono_depth - SPEC_DEPTH_MONO),

	sitem_memory_needed			= 21,
	
	sitem_line_1				= 27,
	sitem_ok_outline			= 28,
	sitem_ignore_comments_alt	= 29,
	
	sitem_scale_popup			= 30
};

enum {
	citem_next			= 1,
	citem_interrupt		= 2,
	citem_next_outline	= 3
};

enum {
	aitem_ok			= 1,
	aitem_ok_outline	= 2,
	aitem_version		= 3,
	aitem_picture		= 4
};

enum {
	pitem_ok				= 1,
	pitem_cancel			= 2,
	
	pitem_cmd_line_first	= 3,
	pitem_cmd_line_ask		= 3,
	pitem_cmd_line_given	= 4,
	pitem_cmd_line_last		= 4,
	pitem_cmd_line_text		= 5,
	
	pitem_console_visible	= 6,
	pitem_specs_visible		= 7,
	
	pitem_next_page_sound	= 8,
	pitem_next_page_window	= 9,
	
	pitem_save_positions	= 10,
	pitem_save_specs		= 11,
	
	pitem_ok_outline		= 15
};



typedef struct {
	DialogPtr			dialog;
	Point				location;
	void				(*hitProc)(DialogPtr, short);
} dialogInfo;

#define NUM_DIALOGS		5
static dialogInfo		theDialogs[NUM_DIALOGS] = { { NULL, { 0 , 0 }, NULL } };

#define confirmInfo		(theDialogs[0])
#define outfileInfo		(theDialogs[1])
#define specInfo		(theDialogs[2])
#define aboutInfo		(theDialogs[3])
#define prefInfo		(theDialogs[4])

static dev_spec			editSpec;
dev_spec				nextSpec = { DEFAULT_PAGE_WIDTH, DEFAULT_PAGE_HEIGHT,
										SPEC_UNITS_INCH, 1,
										DEFAULT_DEV_DPI,
										SPEC_SCALE_PERCENT,
										SPEC_DEPTH_8BIT_COLOR };
										
typedef struct {
	unsigned short		width;
	unsigned short		height;
	unsigned short		units;
} PageSize;

#define units_inches_times_10	('10')
#define units_inches			('in')
#define units_centimeters		('cm')
#define units_millimeters		('mm')
#define units_points			('pt')

static short			**pageScales = NULL;
static short			numPageScales;
static PageSize			**pageSizes = NULL;
static short			numPageSizes;

static PicHandle		animationPict = NULL;
static int				animationFrame;
static unsigned long	animationTime;

#define FRAME_COUNT		10
#define FRAME_OFFSET	24
#define FRAME_RATE		6
#define FRAME_LAG		FRAME_RATE

preferences				thePrefs;
static preferences		editPrefs;


static UserItemUPP		itemButtonOutlineProc = NULL;
static UserItemUPP		itemGrayProc = NULL;
static UserItemUPP		itemAnimationProc = NULL;


static pascal void
itemButtonOutline(WindowPtr theWindow, short item)
{
	short		iType;
	Handle		iHandle;
	Rect		iRect;
	
	GetDItem(theWindow, item, &iType, &iHandle, &iRect);
	PenSize(3, 3);
	PenPat(&qd.black);
	FrameRoundRect(&iRect, 16, 16);
}


static pascal void
itemGray(WindowPtr theWindow, short item)
{
	short		iType;
	Handle		iHandle;
	Rect		iRect;
	
	GetDItem(theWindow, item, &iType, &iHandle, &iRect);
	FillRect(&iRect, &qd.gray);
}


static pascal void
itemAnimation(WindowPtr theWindow, short item)
{
	short		iType;
	Handle		iHandle;
	Rect		iRect, pRect;
	
	if (animationPict == NULL) return;
	
	GetDItem(theWindow, item, &iType, &iHandle, &iRect);
	pRect = (*animationPict)->picFrame;
	OffsetRect(&pRect,
		iRect.left - pRect.left - animationFrame * FRAME_OFFSET,
		iRect.top - pRect.top);
	ClipRect(&iRect);
	DrawPicture(animationPict, &pRect);
	ClipRect(&theWindow->portRect);
}


static void
hiliteItem(DialogPtr d, short item)
{
	short		iType;
	Handle		iHandle;
	Rect		iRect;
	
	GetDItem(d, item, &iType, &iHandle, &iRect);
	HiliteControl((ControlHandle)iHandle, 1);
}

static void
setControl(DialogPtr d, short item, short value)
{
	short		iType;
	Handle		iHandle;
	Rect		iRect;

	GetDItem(d, item, &iType, &iHandle, &iRect);
	if (GetCtlValue((ControlHandle)iHandle) != value)
		SetCtlValue((ControlHandle)iHandle, value);
}

static void
getPosition(DialogPtr d, Point *p)
{
	p->v = d->portRect.top - d->portBits.bounds.top;
	p->h = d->portRect.left - d->portBits.bounds.left;

//{ char m[256]; sprintf(m, "Saving %d, %d\n", p->h, p->v); zcPuts(theConsole, m); }
}

static Boolean
goodPosition(Point *p)
{
	Point		q;
	Boolean		r;
	
	if (p->v == 0  &&  p->h == 0) return false;
	
	q = *p;
	q.h - 8;
	q.v + 4;

	r = PtInRgn(q, GetGrayRgn());

//if (!r) { char m[256]; sprintf(m, "Check %d, %d Failed\n", q.h, q.v); zcPuts(theConsole, m); }
	
	return r;
}

static void
ensureDialogProcs(void)
{
	if (itemButtonOutlineProc == NULL) {
		itemButtonOutlineProc = NewUserItemProc(itemButtonOutline);
		itemGrayProc = NewUserItemProc(itemGray);
		itemAnimationProc = NewUserItemProc(itemAnimation);
		
		if (itemButtonOutlineProc == NULL
		||  itemGrayProc == NULL
		||  itemAnimationProc == NULL)
			noMemoryExit();
	}
}


static void specToDialog(DialogPtr d)
{
	short		iType;
	Handle		iHandle;
	Rect		iRect;
	double		v, h;
	int			i;
	Str255		t, u;
	char		* f;
	
	switch (editSpec.units) {
		case SPEC_UNITS_POINT:	h = editSpec.width;
								v = editSpec.height;
								i = 0;
								break;
								
		case SPEC_UNITS_CM:		h = editSpec.width * (2.54 / 72.0);
								v = editSpec.height * (2.54 / 72.0);
								i = 1;
								break;
		
		default:			
		case SPEC_UNITS_INCH:	h = editSpec.width * (1.0 / 72.0);
								v = editSpec.height * (1.0 / 72.0);
								i = 2;
								break;
	}
								
	sprintf((char *)&t[1], "%.*f", i, v);
	t[0] = strlen((char *)&t[1]);
	GetDItem(d, sitem_size_v, &iType, &iHandle, &iRect);
	GetIText(iHandle, u);
	if (! EqualString(u, t, true, true))
		SetIText(iHandle, t);

	sprintf((char *)&t[1], "%.*f", i, h);
	t[0] = strlen((char *)&t[1]);
	GetDItem(d, sitem_size_h, &iType, &iHandle, &iRect);
	GetIText(iHandle, u);
	if (! EqualString(u, t, true, true))
		SetIText(iHandle, t);
	
	for (i = sitem_first_unit; i <= sitem_last_unit; ++i) 
		setControl(d, i, (i == editSpec.units + sitem_unit_offset) ? 1 : 0);
	
	for (i = 1; i <= numPageSizes; i++) {
		PageSize		* ps;
		float			howClose;
		
		ps = &((*pageSizes)[i - 1]);
		if (ps->units == 0) continue;

		switch (ps->units) {
			case units_inches_times_10:	h = 7.2;		break;
			case units_inches:			h = 72.0;		break;
			case units_centimeters:	h = (72.0 / 2.54);	break;
			case units_millimeters:	h = (72.0 / 25.4);	break;
			default:
			case units_points:			h = 1.0;		break;
		};
		
		howClose = fabs(editSpec.width - (ps->width * h))
				 + fabs(editSpec.height - (ps->height * h));
		if (howClose < 0.01) break;
	}
	if (i > numPageSizes) i = 1;
	GetDItem(d, sitem_size_popup, &iType, &iHandle, &iRect);
	SetCtlValue((ControlHandle)iHandle, i);
	
	setControl(d, sitem_ignore_comments, editSpec.ignore_comments ? 1 : 0);

	switch (editSpec.scale) {
		case SPEC_SCALE_DIVIDE:		h = 72.0 / editSpec.pixels_per_inch;
									i = 3;
									break;
								
		case SPEC_SCALE_TIMES:		h = editSpec.pixels_per_inch / 72.0;
									i = 3;
									break;
		
		case SPEC_SCALE_DPI:		h = editSpec.pixels_per_inch;
									i = 0;
									break;
		
		default:			
		case SPEC_SCALE_PERCENT:	h = editSpec.pixels_per_inch * (100.0 / 72.0);
									i = 0;
									break;
	}

	sprintf((char *)&t[1], "%.*f", i, h);
	t[0] = strlen((char *)&t[1]);
	GetDItem(d, sitem_scale_amount, &iType, &iHandle, &iRect);
	GetIText(iHandle, u);
	if (! EqualString(u, t, true, true))
		SetIText(iHandle, t);
	
	for (i = 1; i <= numPageScales; i++) {
		h = (*pageScales)[i - 1] / 100.0;		
		if (h == editSpec.pixels_per_inch) break;
	}
	if (i > numPageScales) i = 2;
	GetDItem(d, sitem_scale_popup, &iType, &iHandle, &iRect);
	SetCtlValue((ControlHandle)iHandle, i);

	for (i = sitem_first_scale; i <= sitem_last_scale; ++i)
		setControl(d, i, (i == editSpec.scale + sitem_scale_offset) ? 1 : 0);
	
	for (i = sitem_first_depth; i <= sitem_last_depth; ++i)
		setControl(d, i, (i == editSpec.depth + sitem_depth_offset) ? 1 : 0);
	
	h = editSpec.pixels_per_inch / 72.0;
	h = h * h * editSpec.width * editSpec.height;
	switch (editSpec.depth) {
		case SPEC_DEPTH_MONO:			h = h / 8.0;	break;
		case SPEC_DEPTH_4BIT_GRAY:
		case SPEC_DEPTH_4BIT_COLOR:		h = h / 2.0;	break;
		default:
		case SPEC_DEPTH_8BIT_GRAY:
		case SPEC_DEPTH_8BIT_COLOR:	break;
		case SPEC_DEPTH_24BIT_COLOR:	h = h * 4.0;	break;
	}
	if (h < 1024.0)					f = "1k";
	else if (h < (1024.0 * 1024.0))	f = "%.0f k", h = h / 1024.0;
	else							f = "%.1f M", h = h / (1024.0 * 1024.0);
	sprintf((char *)&t[1], f, h);
	t[0] = strlen((char *)&t[1]);
	GetDItem(d, sitem_memory_needed, &iType, &iHandle, &iRect);
	GetIText(iHandle, u);
	if (! EqualString(u, t, true, true))
		SetIText(iHandle, t);
}


static double scaleSize(double f)
{
	switch (editSpec.units) {
		case SPEC_UNITS_POINT:	return f;
		case SPEC_UNITS_CM:		return f * (72.0 / 2.54);
		case SPEC_UNITS_INCH:
		default:				return f * 72.0;			
	}
}

static void
specDialogHit(DialogPtr d, short item)
{
	short		iType;
	Handle		iHandle;
	Rect		iRect;
	double		f;
	float		sIn;
	Str255		t;
	PageSize	*ps;
	int			mi;
	
	GetDItem(d, item, &iType, &iHandle, &iRect);
	
	switch (item) {
		case sitem_ok:
			nextSpec = editSpec;
			// fall through...
			
		case sitem_cancel:
			specInfo.dialog = NULL;
			getPosition(d, &specInfo.location);
			DisposeDialog(d);
			return;	// skip final processing
		
		case sitem_size_v:
			GetIText(iHandle, t);
			t[t[0] + 1] = 0;
			sscanf((char *)&t[1], "%f", &sIn);
			editSpec.height = scaleSize(sIn);
			break;

		case sitem_size_h:
			GetIText(iHandle, t);
			t[t[0] + 1] = 0;
			sscanf((char *)&t[1], "%f", &sIn);
			editSpec.width = scaleSize(sIn);
			break;
		
		case sitem_inch_unit:
		case sitem_cm_unit:
		case sitem_point_unit:
			editSpec.units = item - sitem_unit_offset;
			break;
		
		case sitem_size_popup:
			mi = GetControlValue((ControlHandle)iHandle);
			if (mi > 2) {
				ps = &((*pageSizes)[mi - 1]);
				
				switch (ps->units) {
					case units_inches_times_10:	f = 7.2;	editSpec.units = SPEC_UNITS_INCH;	break;
					case units_inches:			f = 72.0;	editSpec.units = SPEC_UNITS_INCH;	break;
					case units_centimeters:	f = (72.0 / 2.54);	editSpec.units = SPEC_UNITS_CM;	break;
					case units_millimeters:	f = (72.0 / 25.4);	editSpec.units = SPEC_UNITS_CM;	break;
					default:
					case units_points:			f = 1.0;	editSpec.units = SPEC_UNITS_POINT;	break;
				};
									
				editSpec.width = ps->width * f;
				editSpec.height = ps->height * f;
			}
			break;
		
		case sitem_ignore_comments:
		case sitem_ignore_comments_alt:
			editSpec.ignore_comments = ! editSpec.ignore_comments;
			break;
		
		case sitem_scale_amount: {
			double	dpi, idpi, res;
			
			GetIText(iHandle, t);
			if (t[0] == 0) return;	// on empty, let user type!
			
			t[t[0] + 1] = 0;
			sscanf((char *)&t[1], "%f", &sIn);
			
			switch (editSpec.scale) {
				default:
				case SPEC_SCALE_PERCENT:	dpi = sIn * (72.0 / 100.0);	res = 1.0;	break;
				case SPEC_SCALE_TIMES:		dpi = sIn * 72.0;			res = 0.001;break;
				case SPEC_SCALE_DIVIDE:		dpi = 72.0 / sIn;			res = 0.001;break;
				case SPEC_SCALE_DPI:		dpi = sIn;					res = 1.0;	break;
			}
			idpi = floor(dpi + 0.5);
			if (fabs(idpi - dpi) / dpi * sIn / res < 0.35) dpi = idpi;
			editSpec.pixels_per_inch = dpi;
			break;
		}
		
		case sitem_percent_scale:
		case sitem_times_scale:
		case sitem_divide_scale:
		case sitem_dpi_scale:
			editSpec.scale = item - sitem_scale_offset;
			break;
		
		case sitem_scale_popup:
			mi = GetControlValue((ControlHandle)iHandle);
			if (mi > 0) {
				f = (*pageScales)[mi - 1];
				editSpec.pixels_per_inch = f / 100.0;
			}
			break;

		case sitem_mono_depth:
		case sitem_4_bit_gray_depth:
		case sitem_8_bit_gray_depth:
		case sitem_4_bit_color_depth:
		case sitem_8_bit_color_depth:
		case sitem_24_bit_color_depth:
			editSpec.depth = item - sitem_depth_offset;
			break;		
	}
	
	specToDialog(d);
}




void
specDialogOpen(void)
{
	short		iType;
	Handle		iHandle;
	Rect		iRect;
	
	if (specInfo.dialog != NULL) {
		SelectWindow(specInfo.dialog);
		return;
	}
	
	ensureDialogProcs();
	
	if (pageScales == NULL) {
		pageScales = (short **)GetResource('PScl', 303);
		numPageScales = GetHandleSize((Handle)pageScales) / sizeof(short);
		
		pageSizes = (PageSize **)GetResource('PSiz', 302);
		numPageSizes = GetHandleSize((Handle)pageSizes) / sizeof(PageSize);
	}
	
	editSpec = nextSpec;
	specInfo.dialog = GetNewDialog(spec_resource, NULL, (WindowPtr)-1);
	specInfo.hitProc = specDialogHit;
	
	// set up user items here...
	GetDItem(specInfo.dialog, sitem_line_1, &iType, &iHandle, &iRect);
	SetDItem(specInfo.dialog, sitem_line_1,
		userItem, (Handle)itemGrayProc, &iRect);
	GetDItem(specInfo.dialog, sitem_ok_outline, &iType, &iHandle, &iRect);
	SetDItem(specInfo.dialog, sitem_ok_outline,
		userItem, (Handle)itemButtonOutlineProc, &iRect);
	
	// inactivate controls we don't use
	GetDItem(specInfo.dialog, sitem_8_bit_gray_depth, &iType, &iHandle, &iRect);
	HiliteControl((ControlHandle)iHandle, 255);
	GetDItem(specInfo.dialog, sitem_4_bit_color_depth, &iType, &iHandle, &iRect);
	HiliteControl((ControlHandle)iHandle, 255);
	GetDItem(specInfo.dialog, sitem_24_bit_color_depth, &iType, &iHandle, &iRect);
	HiliteControl((ControlHandle)iHandle, 255);
	
	specToDialog(specInfo.dialog);
	
	if (goodPosition(&specInfo.location)) {
		MoveWindow(specInfo.dialog,
			specInfo.location.h, specInfo.location.v, false);
	}
	
	ShowWindow(specInfo.dialog);
}

static void prefToDialog(DialogPtr d)
{
	short		iType;
	Handle		iHandle;
	Rect		iRect;
	int			i;
	Str255		u;
	
	
	for (i = pitem_cmd_line_first; i <= pitem_cmd_line_last; i += 1)
		setControl(d, i, (i == pitem_cmd_line_ask) == editPrefs.command_ask);

	GetDItem(d, pitem_cmd_line_text, &iType, &iHandle, &iRect);
	GetIText(iHandle, u);
	if (! EqualString(u, editPrefs.command_line, true, true))
		SetIText(iHandle, editPrefs.command_line);

	setControl(d, pitem_console_visible,	editPrefs.console_visible ? 1 : 0);
	setControl(d, pitem_specs_visible,		editPrefs.specs_visible ? 1 : 0);

	setControl(d, pitem_next_page_sound,	editPrefs.next_page_sound ? 1 : 0);
	setControl(d, pitem_next_page_window,	editPrefs.next_page_window ? 1 : 0);

	setControl(d, pitem_save_positions,		editPrefs.save_positions ? 1 : 0);
	setControl(d, pitem_save_specs,			editPrefs.save_specs ? 1 : 0);
}



static void
prefDialogHit(DialogPtr d, short item)
{
	short		iType;
	Handle		iHandle;
	Rect		iRect;
	
	GetDItem(d, item, &iType, &iHandle, &iRect);
	
	switch (item) {
		case pitem_ok:
			thePrefs = editPrefs;
			savePreferences(false);
			// fall through...
			
		case pitem_cancel:
			prefInfo.dialog = NULL;
			getPosition(d, &prefInfo.location);
			DisposeDialog(d);
			return;	// skip final processing
		
		case pitem_cmd_line_ask:
			editPrefs.command_ask = true;
			editPrefs.console_visible = true;
			break;
		
		case pitem_cmd_line_given:
			editPrefs.command_ask = false;
			break;
			
		case pitem_cmd_line_text:
			GetIText(iHandle, editPrefs.command_line);
			editPrefs.command_ask = false;
			break;

		case pitem_console_visible:
			editPrefs.console_visible = ! editPrefs.console_visible;
			if (editPrefs.console_visible == false)
				editPrefs.command_ask = false;
			break;
		
		case pitem_specs_visible:
			editPrefs.specs_visible = ! editPrefs.specs_visible;
			break;
		
		case pitem_next_page_sound:
			editPrefs.next_page_sound = ! editPrefs.next_page_sound;
			break;
		
		case pitem_next_page_window:
			editPrefs.next_page_window = ! editPrefs.next_page_window;
			break;
		
		case pitem_save_positions:
			editPrefs.save_positions = ! editPrefs.save_positions;
			break;
		
		case pitem_save_specs:
			editPrefs.save_specs = ! editPrefs.save_specs;
			break;
	}
	
	prefToDialog(d);
}




void
prefDialogOpen(void)
{
	short		iType;
	Handle		iHandle;
	Rect		iRect;
	
	if (prefInfo.dialog != NULL) {
		SelectWindow(prefInfo.dialog);
		return;
	}
	
	ensureDialogProcs();
			
	editPrefs = thePrefs;
	prefInfo.dialog = GetNewDialog(pref_resource, NULL, (WindowPtr)-1);
	prefInfo.hitProc = prefDialogHit;
	

	// set up user items here...
	GetDItem(prefInfo.dialog, pitem_ok_outline, &iType, &iHandle, &iRect);
	SetDItem(prefInfo.dialog, pitem_ok_outline,
		userItem, (Handle)itemButtonOutlineProc, &iRect);

	prefToDialog(prefInfo.dialog);
	
	if (goodPosition(&prefInfo.location)) {
		MoveWindow(prefInfo.dialog,
			prefInfo.location.h, prefInfo.location.v, false);
	}
	
	ShowWindow(prefInfo.dialog);
}

static const char * prefFileName = (char *)"\pMac GS Viewer Prefs";

void
loadPreferences(void)
{
	FSSpec			pFile;
	short			ref;
	long			count;
	OSErr			err;
	
	err = FindFolder(kOnSystemDisk, kPreferencesFolderType, kCreateFolder,
			&pFile.vRefNum, &pFile.parID);
	if (err != noErr) goto failed;
	
	memcpy(pFile.name, prefFileName, prefFileName[0] + 1);
	err = FSpOpenDF(&pFile, fsRdPerm, &ref);
	if (err != noErr) goto failed;
	
	count = sizeof(preferences);
	err = FSRead(ref, &count, &thePrefs);
	FSClose(ref);
	
	if (err != noErr
	||  count != sizeof(preferences)
	||  thePrefs.version != PREFS_VERSION)
		goto failed;

	// we were successful
	if (thePrefs.save_positions) {
		confirmInfo.location = thePrefs.next_page_position;
		specInfo.location = thePrefs.specs_position;
		prefInfo.location = thePrefs.prefs_position;
	}
	
	if (thePrefs.save_specs) {
		nextSpec = thePrefs.init_spec;
	}
	
	return;

failed:
	memset(&thePrefs, 0, sizeof(thePrefs));
	
	thePrefs.version = PREFS_VERSION;
	thePrefs.save_positions = true;
	thePrefs.save_specs = true;
	//memset(&thePrefs.command_line, 0, sizeof(thePrefs.command_line);
	thePrefs.console_visible = true;
	//thePrefs.console_position.h = thePrefs.console_posistion.v = 0;
	//thePrefs.console_size.h = thePrefs.console_size.v = 0;
	thePrefs.next_page_window = true;
	thePrefs.next_page_sound = true;
	//thePrefs.next_page_position.h = thePrefs.next_page_posistion.v = 0;
	//thePrefs.specs_visible = false;
	//thePrefs.specs_position.h = thePrefs.specs_posistion.v = 0;
	thePrefs.init_spec = nextSpec;
}

void
savePreferences(Boolean atQuit)
{
	FSSpec			pFile;
	short			ref;
	long			count;
	OSErr			err;
		
	if (atQuit
	&&  ! thePrefs.save_positions
	&&  ! thePrefs.save_specs)
		return;

	if (thePrefs.save_positions) {
		zcCreateSpec	s;
		
		s.title = NULL;	// we don't care
		zcGetSpec(theConsole, &s);
		thePrefs.console_position = s.loc;
		thePrefs.console_size.h = s.cols;
		thePrefs.console_size.v = s.rows;
		
		if (confirmInfo.dialog != NULL)
			getPosition(confirmInfo.dialog, &confirmInfo.location);
		if (specInfo.dialog != NULL)
			getPosition(specInfo.dialog, &specInfo.location);
		if (prefInfo.dialog != NULL)
			getPosition(prefInfo.dialog, &prefInfo.location);

		thePrefs.next_page_position = confirmInfo.location;
		thePrefs.specs_position = specInfo.location;
		thePrefs.prefs_position = prefInfo.location;
	}
	
	if (thePrefs.save_specs) {
		thePrefs.init_spec = nextSpec;
	}
	
	err = FindFolder(kOnSystemDisk, kPreferencesFolderType, kCreateFolder,
			&pFile.vRefNum, &pFile.parID);
	if (err != noErr) goto failed;
	
	memcpy(pFile.name, prefFileName, prefFileName[0] + 1);
	
	err = FSpCreate(&pFile, 'gsVR', 'pref', smSystemScript);
	if (err != noErr  &&  err != dupFNErr) goto failed;
	
	err = FSpOpenDF(&pFile, fsWrPerm, &ref);
	if (err != noErr) goto failed;
	
	count = sizeof(preferences);
	err = FSWrite(ref, &count, &thePrefs);
	FSClose(ref);
	
	if (err != noErr
	||  count != sizeof(preferences))
		goto failed;

failed:
	return;
}


static void
confirmDialogHit(DialogPtr d, short item)
{
	short		iType;
	Handle		iHandle;
	Rect		iRect;
	
	GetDItem(d, item, &iType, &iHandle, &iRect);
	
	switch (item) {
		case citem_interrupt:
			interruptRequest = true;
			// fall through...
			
		case citem_next:
			note_interp_state(INTERP_RUN);
			
			confirmInfo.dialog = NULL;
			getPosition(d, &confirmInfo.location);
			DisposeDialog(d);
			break;
		
		default:
			break;
	}
}

void
confirmDialogDone(Boolean interrupt)
{
	if (confirmInfo.dialog != NULL) {
		int		i;
		
		i = interrupt ? citem_interrupt : citem_next;
		
		hiliteItem(confirmInfo.dialog, i);
		confirmDialogHit(confirmInfo.dialog, i);
	}
}


void
confirmDialogOpen()
{
	short		iType;
	Handle		iHandle;
	Rect		iRect;
	
	if (confirmInfo.dialog != NULL) {
		SelectWindow(confirmInfo.dialog);
		return;
	}
	
	ensureDialogProcs();
	
	confirmInfo.dialog = GetNewDialog(confirm_resource, NULL, (WindowPtr)-1);
	confirmInfo.hitProc = confirmDialogHit;
	
	// set up user items here...
	GetDItem(confirmInfo.dialog, citem_next_outline, &iType, &iHandle, &iRect);
	SetDItem(confirmInfo.dialog, citem_next_outline,
		userItem, (Handle)itemButtonOutlineProc, &iRect);
	
	if (! goodPosition(&confirmInfo.location)) {
		confirmInfo.location.h = gWidth - 167;
		confirmInfo.location.v = LMGetMBarHeight() + 26;
	}
	MoveWindow(confirmInfo.dialog,
		confirmInfo.location.h, confirmInfo.location.v, false);
	
	ShowWindow(confirmInfo.dialog);
}

static void
aboutDialogHit(DialogPtr d, short item)
{
	switch (item) {
		case aitem_ok:
			aboutInfo.dialog = NULL;
			DisposeDialog(d);
			break;
		
		default:
			break;
	}
}

static void
aboutDialogAnimate(void)
{
	GrafPtr		savePort;
	int			thisFrame;

	thisFrame = (TickCount() / FRAME_RATE) % FRAME_COUNT;
	if (thisFrame == animationFrame) return;
	animationFrame = thisFrame;
	
	GetPort(&savePort);
	SetPort(aboutInfo.dialog);
	itemAnimation(aboutInfo.dialog, aitem_picture);
	SetPort(savePort);
}

void
aboutDialogOpen(void)
{
	Handle			v1, v2;
	char			s1[100], s2[100];
	Str255			message;

	short		iType;
	Handle		iHandle;
	Rect		iRect;
	EventRecord	e;
	
	if (aboutInfo.dialog != NULL) {
		SelectWindow(aboutInfo.dialog);
		return;
	}
	
	v1 = GetResource('vers', 1);	HLock(v1);
	v2 = GetResource('vers', 2);	HLock(v2);
	
	strncpy(s1, *v1 + 7, (*v1)[6]);
	s1[(*v1)[6]] = '\0';
	strncpy(s2, *v2 + 7, (*v2)[6]);
	s2[(*v2)[6]] = '\0';
	
	sprintf((char *)&(message[1]), "Ghostscript %s\rMac GS Viewer %s", s2, s1);
	message[0] = strlen((char *)&(message[1]));
	
	HUnlock(v1);	ReleaseResource(v1);
	HUnlock(v2);	ReleaseResource(v2);

	ensureDialogProcs();
	if (animationPict == NULL)
		animationPict = GetPicture(animation_resource);
	animationFrame = 0;
	animationTime = TickCount() + FRAME_LAG;
	
	aboutInfo.dialog = GetNewDialog(about_resource, NULL, (WindowPtr)-1);
	aboutInfo.hitProc = aboutDialogHit;
	
	// set up user items here...
	GetDItem(aboutInfo.dialog, aitem_ok_outline, &iType, &iHandle, &iRect);
	SetDItem(aboutInfo.dialog, aitem_ok_outline,
		userItem, (Handle)itemButtonOutlineProc, &iRect);

	GetDItem(aboutInfo.dialog, aitem_picture, &iType, &iHandle, &iRect);
	SetDItem(aboutInfo.dialog, aitem_picture,
		userItem, (Handle)itemAnimationProc, &iRect);
	
	GetDItem(aboutInfo.dialog, aitem_version, &iType, &iHandle, &iRect);
	SetIText(iHandle, message);
	
	ShowWindow(aboutInfo.dialog);
	
	while (aboutInfo.dialog != NULL) {
		aboutDialogAnimate();
		
		WaitNextEvent(everyEvent, &e, FRAME_RATE / 2, NULL);
		ViewAppEvent(&e);
	}
}


int dialogHandleEvent(EventRecord * e)
{
	WindowPtr		window;
	short			item;
	Boolean			doDefault;
	char			keyHit;
	short			d;
	
	for (d = 0; d < NUM_DIALOGS; d++)
		if (theDialogs[d].dialog != NULL) break;
	if (d == NUM_DIALOGS) return 0;	// no dialogs up
		
	doDefault = false;
	
	switch (e->what) {
		case keyDown:
		case autoKey:
			if (e->modifiers & cmdKey) return 0;
			keyHit = e->message & charCodeMask;
			doDefault = keyHit == 0x0D	// return key
					 || keyHit == 0x03;	// enter key
			if (doDefault) {
				window = FrontWindow();
				break;
			}
			// else fall through

		default:
			if (! IsDialogEvent(e)) return 0;
			if (! DialogSelect(e, &window, &item)) return 1;
	}
	
	for (d = 0; d < NUM_DIALOGS; d++)
		if (theDialogs[d].dialog == window) break;
	if (d == NUM_DIALOGS) return 0;	// someone else's dialog

	if (doDefault) {
		item = 1;
		hiliteItem(window, item);
	}
	
	if (theDialogs[d].hitProc != NULL)
		theDialogs[d].hitProc(window, item);

	return 1;
}
