# Copyright (C) 1989 - 1995 Aladdin Enterprises.  All rights reserved.
# 
# This file is part of Aladdin Ghostscript.
# 
# Aladdin Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author
# or distributor accepts any responsibility for the consequences of using it,
# or for whether it serves any particular purpose or works at all, unless he
# or she says so in writing.  Refer to the Aladdin Ghostscript Free Public
# License (the "License") for full details.
# 
# Every copy of Aladdin Ghostscript must include a copy of the License,
# normally in a plain ASCII text file named PUBLIC.  The License grants you
# the right to copy, modify and redistribute Aladdin Ghostscript, but only
# under certain conditions described in the License.  Among other things, the
# License requires that the copyright notice and this notice be preserved on
# all copies.

# Header of makefile for Macintosh configuration.

# ------------------------------- Options ------------------------------- #

####### The following are the only parts of the file you should need to edit.

# ------ Required Enviornment Variables ------ #

# There are a number of enviornment variables that must be set before
# running this makefile.  See the file "Worksheet" that is with the
# Macintosh specific sources for how to set them up.


# ------ Generic options ------ #

# Define the default directory/ies for the runtime
# initialization and font files.  Separate multiple directories with a ,.
# Note that if you want the interpreter to search the current directory,
# you must include ':' explicitly.

GS_LIB_DEFAULT=:,:files,:fonts,:examples

# Define the location of the documentation files.

GS_DOCDIR=:files

# Define the name of the interpreter initialization file.
# (There is no reason to change this.)

GS_INIT=gs_init.ps

# Choose generic configuration options.

# -DDEBUG
#	includes debugging features (-Z switch) in the code.
#	  Code runs substantially slower even if no debugging switches
#	  are set.
# -DNOPRIVATE
#	makes private (static) procedures and variables public,
#	  so they are visible to the debugger and profiler.
#	  No execution time or space penalty.

#GENOPT=-d DEBUG
GENOPT=

# Define the name of the executable file.

GS="Mac GS Viewer"

# Define the directory where the IJG JPEG library sources are stored.
# (There is no reason to change this, it is set by an environment variable)

JSRCDIR={gsjpeg}



# ------ Platform-specific options ------ #

# Define the name of the C compiler.

CC=MWC68K
# CC=MWCPPC

# Define the name of the linker for the final link step.

CCLD=MWLink68K
# CCLD=MWLinkPPC

# Define the default compiler flags.
CDEFS=-d CHECK_INTERRUPTS -d __MACINTOSH__

CCW_FLAGS=-enum int -nosyspath -notonce -trigraphs on 
CCW_68K=-model far -intsize 4
CCW_68K_OPT=-opt all
CCW_PPC=-str readonly
CCW_PPC_OPT=-opt all,schedule,l4
CCW_DEBUG=-sym on -sym fullpath -mbg full

#CFLAGS={CDEFS} {CCW_FLAGS} {CCW_68K} {CCW_DEBUG}
CFLAGS={CDEFS} {CCW_FLAGS} {CCW_68K} {CCW_68K_OPT} 

CCINCLUDES=-i "{gsmac}" -i "{gssrc}" -i "{gsjpeg}"

# Define the link options

LDCW_FLAGS=-t 'APPL' -c 'gsVR' -rsrcmerge
LDCW_LIBS="{MW68KLibraries}ANSI (4i) C.68K.Lib" 
	"{MW68KLibraries}MacOS.lib" 
	"{MW68KLibraries}MathLib68K (4i).Lib"
LDCW_DEBUG=-sym full -sym fullpath -map {GS}.MAP -osym {GS}.SYM

#LDFLAGS={LDCW_FLAGS} {LDCW_DEBUG}
LDFLAGS={LDCW_FLAGS}
LDLIBS={LDCW_LIBS}

# Define the commands to build MPW tools

CCAUX=MWC68K -notonce -opt all -intsize 4 {CCINCLUDES}
CCLDAUX=MWLink68K -mpwtool
AUXLIBS="{MW68KLibraries}MPW ANSI (4i) C.68K.Lib" 
	"{MW68KLibraries}MacOS.lib"



# Define whether this platform has floating point hardware:
#	FPU_TYPE=2 means floating point is faster than fixed point.
# (This is the case on some RISCs with multiple instruction dispatch.)
#	FPU_TYPE=1 means floating point is at worst only slightly slower
# than fixed point.
#	FPU_TYPE=0 means that floating point may be considerably slower.
#	FPU_TYPE=-1 means that floating point is always much slower than
# fixed point.

FPU_TYPE=1		# for 68k
# FPU_TYPE=2	# for PPC



# ------ Devices and features ------ #

# Choose the language feature(s) to include.  See gs.mak for details.

FEATURE_DEVS=level2.dev

# Choose the device(s) to include.  See devs.mak for details.

DEVICE_DEVS=mac.dev
DEVICE_DEVS1=
DEVICE_DEVS2=
DEVICE_DEVS3=
DEVICE_DEVS4=cdj550.dev
DEVICE_DEVS5=
DEVICE_DEVS6=stcolor.dev
DEVICE_DEVS7=
DEVICE_DEVS8=pcxmono.dev pcxgray.dev pcx16.dev pcx256.dev pcx24b.dev
DEVICE_DEVS9=pbm.dev pbmraw.dev pgm.dev pgmraw.dev ppm.dev ppmraw.dev
DEVICE_DEVS10=tifflzw.dev tiffpack.dev
DEVICE_DEVS11=
DEVICE_DEVS12=
DEVICE_DEVS13=
DEVICE_DEVS14=
DEVICE_DEVS15=

# ---------------------------- End of options --------------------------- #

# Define the name of the makefile -- used in dependencies.

MAKEFILE=makefile

# Define the ANSI-to-K&R dependency.
# (MPW & CW compiliers accept ANSI syntax.)

AK=

# Define the compilation rules and flags.

CCC={CC} {CCFLAGS} {CCINCLUDES}


# ----------------------------- Generic stuff ----------------------------- #

# The remainder of this file sets up variables used by the generic makefiles.
# There is little reason to change these.

# Define the platform name.

PLATFORM=mac_

# Define the syntax for command, object, and executable files.

O=-o 
OBJ=o
XE=

# Define the current directory prefix and shell invocations.

D=
EXP=
EXPP=
SHELL=
SH={SHELL}
SHP={SH} {EXP}

# Define the arguments for genconf.

CONFILES=-p "%s" -pl "%s" -pL "%s" -ol ld.tr

# Define the compilation rules and flags.

CCFLAGS={GENOPT} {CFLAGS}

.{OBJ}  .c
	{CCC} {DepDir}{Default}.c -o {TargDir}{Default}.o

":"  "{gsmac}" "{gssrc}" "{gsjpeg}"

CCCF={CCC}
CCD={CCC}
CCINT={CCC} -o interp.{OBJ}

# Define variables that this build does not need, but that MPW
# make complains about no being defined:

MM=
PCFBASM=
BGIDIR=
USE_ASM=

# --------------------------- Remainder makefile ---------------------------- #

# The middle of the makefile (gs.mak, devs.mak, jpeg.mak) is generic.
# The file Workspace has scripts for converting these makefile to MPW
# format and then concatenating them with this file and mac-tail.mak.

