/* Copyright (C) 1989 - 1995 Aladdin Enterprises.  All rights reserved.
  
  This file is part of Aladdin Ghostscript.
  
  Aladdin Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No author
  or distributor accepts any responsibility for the consequences of using it,
  or for whether it serves any particular purpose or works at all, unless he
  or she says so in writing.  Refer to the Aladdin Ghostscript Free Public
  License (the "License") for full details.
  
  Every copy of Aladdin Ghostscript must include a copy of the License,
  normally in a plain ASCII text file named PUBLIC.  The License grants you
  the right to copy, modify and redistribute Aladdin Ghostscript, but only
  under certain conditions described in the License.  Among other things, the
  License requires that the copyright notice and this notice be preserved on
  all copies.
*/

/* math_.h */
/* Generic substitute for math.h */

/* We must include std.h before any file that includes sys/types.h. */
#include "std.h"

#ifdef VMS
/*  DEC VAX/VMS C comes with a math.h file, but GNU VAX/VMS C does not. */
#  include "vmsmath.h"
#elif defined(__MWERKS__) && defined(powerc) && !defined(__TYPES__)
/*	Metrowerks' math.h includes fp.h, which includes Types.h which brings in
    all of the Macintosh headers!  Therefore we fool it a bit with our own
    .h file which is really just a copy of the needed math things from Types.h. */
#  define __TYPES__
#  include <math_types_.h>
#  include <math.h>
#  undef __TYPES__
#else
#  include <math.h>
#endif

/* math.h is different for Turbo and Unix.... */
#ifndef M_PI
#  ifdef PI
#    define M_PI PI
#  else
#    define M_PI 3.14159265358979324
#  endif
#endif

/* Factors for converting between degrees and radians */
#define degrees_to_radians (M_PI / 180.0)
#define radians_to_degrees (180.0 / M_PI)

/* Define the hypot procedure on those few systems that don't provide it. */
#if defined(_IBMR2) || defined(MACINTOSH)
/* The RS/6000 has hypot, but math.h doesn't declare it! */
/* Same for Macintosh math.h files */
extern double hypot(double, double);
#else
#  if !defined(__TURBOC__) && !defined(BSD4_2) && !defined(VMS)  && !defined(MACINTOSH)
#    define hypot(x,y) sqrt((x)*(x)+(y)*(y))
#  endif
#endif

#ifdef OSK
/* OSK has atan2 and ldexp, but math.h doesn't declare them! */
extern double atan2(), ldexp();
#endif

#ifdef DEBUG

/* Intercept calls on sqrt for debugging. */
extern double gs_sqrt(P3(double, const char _ds *, int));
#define sqrt(x) gs_sqrt(x, __FILE__, __LINE__)

#endif				/* DEBUG */
