/* zarfcon.h

	ZarfCon is a text console package for Macintosh.  It was born out of
	a need for a good generic text console package that could be used on
	the Mac with a minimum of work across different compilers.  Most console
	packages that come with Mac development environments fail this test
	because they are heavily dependent on the implementation of the ANSI
	file library, do not interact well with other Mac programming, and
	(at least on some) use 680x0 code! (You know who you are...)

	This package does NOT try to automatically intercept stdin, stdout,
	or stderr.  Instead, it provides a generally useful text input and
	output window which you can write to using calls that are like (though
	different) from the ANSI character stream i/o calls.  If you had the
	sources for the ANSI library, you could integrate this package so that
	stdin, stdout and stderr used it by default (hint, hint).
	
	
	Copyright (C) 1994 Glyphic Technology
	
	Permission to use, copy, modify, distribute, and sell this software and
	its documentation for any purpose is hereby granted without fee, provided
	that the above copyright notice and this permission notice appear in all
	copies of the software and related documentation.
 */

#ifndef __zarfcon_h__
#define __zarfcon_h__

//#include <stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef int zcConsole;			// a reference to a console
#define zcNoConsole		(-1)
#define zcAllConsoles	(-2)

#define zcOK			(0)
#define zcEOF			(-1)
#define zcNoInput		(-2)

typedef struct {
	Point	loc;				// where to put it
	int		rows, cols;			// how big is it
	int		font, size;			// font parameters
	char	* title;
	int		echo:1;				// echo input
	int		raw:1;				// no buffering or lineediting
	int		delay:1;			// delay until input on zcGet*()
//	int		noinput:1;			// accept no input, not yet implemented
	int		visible:1;			// visible
	int		growable:1;			// user can change its shape
	int		closeable:1;		// user can close window (really only hides it!)
} zcCreateSpec;

typedef void zcEventMgrProc(void);

void zcInit(zcEventMgrProc *mgr);
void zcExit(void);

void zcDefaultSpec(zcCreateSpec *s);
zcConsole zcCreate(zcCreateSpec *s);
void zcDestroy(zcConsole c);
void zcGetSpec(zcConsole c, zcCreateSpec *s);

WindowPtr zcWindow(zcConsole c);

int zcHandleEvent(EventRecord *e);
void zcHouseKeeping(void);

//int zcPrintf(zcConsole c, const char *format, ...);
//int zcScanf(zcConsole c, const char *format, ...);
//int zcVPrintf(zcConsole c, const char *format, va_list args);

/* This is the rational set of calls..
	These all return:
		zcEOF:		if end of file (console is closed)
		zcNoInput:	there is nothing to get (and delay is false)
 */
int zcGetc(zcConsole c);
	// returns: char

int zcGets(zcConsole c, char *string, int nchars);
	// returns: n chars read, string is always null terminated
	
int zcGetbuf(zcConsole c, char *buffer, int nchars);
	// returns: n chars read, string is not null terminated
	
int zcPutc(zcConsole c, int ch);
int zcPuts(zcConsole c, const char *string);
int zcPutbuf(zcConsole c, const char *string, int nchars);

int zcTypec(zcConsole c, int ch);
int zcTypes(zcConsole c, const char *string);
int zcTypebuf(zcConsole c, const char *string, int nchars);

void zcFlush(zcConsole c);

/* This is the ANSI like set of calls...
	These are defined to match the ANSI 'f' calls...
 */


#ifdef __cplusplus
}
#endif

#endif